import numpy as np
import tkinter
import io
import cv2
from PIL import Image

def get_binary(img, gauss=(1,1), threshold=235):
    ''' img -> grayscale -> binary '''
    gray_img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    img_preprocessed = cv2.GaussianBlur(gray_img, gauss, 0)
    _, img_binary = cv2.threshold(img_preprocessed, threshold, 255, cv2.THRESH_BINARY)
    img_binary = cv2.bitwise_not(img_binary)
    return img_binary

def get_good_img(img, epsilon=5):
    gray_img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    row, col = gray_img.shape
    flatten_gray_img = gray_img.reshape(row*col)
    av = np.mean(flatten_gray_img)
    va = np.std(flatten_gray_img)
    img = get_binary(img, threshold = av-va+epsilon)
    return img

def padding(im_, row_pad = None, col_pad = None):
    row,column = im_.shape
    ## padding zeros ##
    zeros_down = np.zeros(row_pad*column).reshape(row_pad, column)
    zeros_up = np.zeros(row_pad*column).reshape(row_pad, column)
    im_ = np.append(im_, zeros_down, axis=0)
    im_ = np.append(zeros_up, im_, axis=0)
    im_ = np.transpose(im_)
    zeros_down = np.zeros(col_pad*(2*row_pad + row)).reshape(col_pad, 2*row_pad + row)
    zeros_up = np.zeros(col_pad*(2*row_pad + row)).reshape(col_pad, 2*row_pad + row)
    im_ = np.append(im_, zeros_down, axis=0)
    im_ = np.append(zeros_up, im_, axis=0)
    im_ = np.transpose(im_)
    return im_

def get28image(img_path, yohaku = 3, min_img_area = 30, e=5, showing=False):
    # get_binary is defined above.
    img = cv2.imread(img_path)
    img2binary = get_good_img(img, epsilon=e)
    _, img_contours, _ = cv2.findContours(img2binary, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
    # getting images of characters as a list "large_contours"
    large_contours = [cnt for cnt in img_contours if cv2.contourArea(cnt) > min_img_area]
    # sort "large_contours" along horizontal axis of "img"
    chars = []
    imgs = []
    xs = []
    for contour in large_contours:
        x, y, w, h = cv2.boundingRect(contour) # (x,y)=upleft edge coordinate, width, hight
        im = get_good_img(img, epsilon=e)
        im_ = im[y:y+h,x:x+w] # character image
        ### making rectangler image to square image
        row, column = im_.shape
        if row > column:
            im_ = padding(im_, row_pad=0, col_pad=(row-column)//2)
        else:
            im_ = padding(im_, row_pad=(column-row)//2, col_pad=0)
        ### scale square image to 28-x28- pre-image
        pilImg = Image.fromarray(im_)
        imgs.append(pilImg)
        pilImg_pre28 = pilImg.resize((28-2*yohaku,28-2*yohaku))
        preimg_28 = np.asarray(pilImg_pre28)
        ### padding pre-image to 28x28 image
        img_28 = padding(preimg_28, row_pad=yohaku, col_pad=yohaku)
        if showing:
            plt.imshow(img_28)
            plt.show()
        ## append "img_28" to "chars", "x" to "xs" ##
        chars.append(img_28)
        xs.append(x)
    return img_28

class Scribble:
    def on_pressed(self, event):
        self.sx = event.x
        self.sy = event.y
        self.canvas.create_oval(self.sx, self.sy, event.x, event.y, outline = "black", width = 10)
    
    def on_dragged(self, event):
        self.canvas.create_line(self.sx, self.sy, event.x, event.y, fill = "black", width = 10)
        self.sx = event.x
        self.sy = event.y
    
    def save_image(self, event):
        ps = self.canvas.postscript(colormode = 'color')
        im = Image.open(io.BytesIO(ps.encode('utf-8')))
        im.save('canvas.jpg')
    
    def __init__(self):
        window = tkinter.Tk()
        self.canvas = tkinter.Canvas(window, bg = "white", width = 300, height = 300)
        self.canvas.pack()
        
        self.canvas.bind("<ButtonPress-1>", self.on_pressed)
        self.canvas.bind("<B1-Motion>", self.on_dragged)
        self.canvas.bind("<Leave>", self.save_image)
        
        window.mainloop()

